@@ -4,7 +4,7 @@
 
 ;; Author: Philip Kaludercic <philipk@posteo.net>
 ;; Maintainer: Philip Kaludercic <philipk@posteo.net>
-;; Version: 1.0.0
+;; Version: 1.0.1
 ;; Package-Requires: ((emacs "26.1"))
 ;; Keywords: lisp, local
 ;; URL: https://git.sr.ht/~pkal/setup
@@ -43,10 +43,10 @@
 ;;      (unless global-auto-revert-mode
 ;;        (:hook auto-revert-mode)))
 ;;
-;;    (setup (:package paredit)
-;;      (:hide-mode)
-;;      (setq paredit-override-check-parens-function (lambda (_) t))
-;;      (:hook-into scheme-mode lisp-mode))
+;;    (setup (:package company)
+;;      (:bind "M-TAB" company-complete)
+;;      (:hook-into prog-mode conf-mode))
+
 
 ;; expanded to the to the functional equivalent of
 
@@ -64,14 +64,12 @@
 ;;    (unless global-auto-revert-mode
 ;;      (add-hook 'dired-mode-hook #'auto-revert-mode))
 ;;
-;;    (unless (package-install-p 'paredit)
-;;      (package-install 'paredit))
-;;    (setq minor-mode-alist
-;;          (delq (assq 'paredit-mode minor-mode-alist)
-;;                minor-mode-alist))
-;;    (setq paredit-override-check-parens-function (lambda (_) t))
-;;    (add-hook 'scheme-mode-hook #'paredit-mode)
-;;    (add-hook 'lisp-mode-hook #'paredit-mode)
+;;    (unless (package-install-p 'company)
+;;      (package-install 'company))
+;;    (with-eval-after-load 'company
+;;      (define-key company-mode-map (kbd "M-TAB") #'company-complete))
+;;    (add-hook 'prog-mode-hook #'company-mode)
+;;    (add-hook 'conf-mode-hook #'company-mode)
 
 ;; Additional "keywords" can be defined using `setup-define'.  All
 ;; known keywords are documented in the docstring for `setup'.
@@ -83,14 +81,14 @@
 
 (require 'elisp-mode)
 
-(defconst setup--quit-sym (gensym)
-  "Symbol thrown on an early exit.")
-
 (defvar setup--need-quit)               ;see `setup-quit'
 
-(defvar setup-opts `((quit . ,setup--quit-sym))
-  "Alist defining context-specific options.
-Values are extracted using `setup-get'.")
+(defvar setup-opts `((quit . ,(make-symbol "setup-quit")))
+  "Alist defining the context for local macros.
+Context-modifying macros (`:with-feature', `:with-mode', ...)
+prepend the new context to this variable using `let', before
+calling `setup-expand'.  Within the macro definitions `setup-get'
+is used to retrieve the current context.")
 
 (defvar setup-macros nil
   "Local macro definitions to be bound in `setup' bodies.
@@ -140,7 +138,7 @@ NAME may also be a macro, if it can provide a symbol."
                  (macroexp-progn body))
                (append setup-macros macroexpand-all-environment))))
     (if setup--need-quit
-        `(catch ',setup--quit-sym ,@(macroexp-unprogn res))
+        `(catch ',(setup-get 'quit) ,@(macroexp-unprogn res))
       res)))
 
 ;;;###autoload
@@ -234,10 +232,12 @@ If not given, it is assumed nothing is evaluated."
              #'setup-xref-def-function)
 
 
-;;; common utility functions for keywords
+;;; Common utility functions for keywords
 
 (defun setup-get (opt)
-  "Return value for OPT."
+  "Retrieve the context-sensitive value for OPT.
+If the context is not defined, an error is thrown.  See
+`setup-opts' for more details."
   (or (cdr (assq opt setup-opts))
       (error "Cannot deduce %S from context" opt)))
 
@@ -301,7 +301,7 @@ and VAL into one s-expression."
         ((error "Invalid option %S" name))))
 
 
-;;; definitions of `setup' keywords
+;;; Definitions of `setup' keywords
 
 (setup-define :with-feature
   (lambda (features &rest body)
@@ -376,7 +376,7 @@ If HOOK is a list, apply BODY to all elements of HOOK."
          (package-refresh-contents))
        (package-install ',package)))
   :documentation "Install PACKAGE if it hasn't been installed yet.
-This macro can be used as HEAD, and it will replace itself with
+This macro can be used as NAME, and it will replace itself with
 the first PACKAGE."
   :repeatable t
   :shorthand #'cadr)
@@ -386,7 +386,7 @@ the first PACKAGE."
     `(unless (require ',feature nil t)
        ,(setup-quit)))
   :documentation "Try to require FEATURE, or stop evaluating body.
-This macro can be used as HEAD, and it will replace itself with
+This macro can be used as NAME, and it will replace itself with
 the first FEATURE."
   :repeatable t
   :shorthand #'cadr)
@@ -522,7 +522,7 @@ supported:
   (lambda (hook function)
     `(add-hook ',(setup-get 'hook)
                (lambda ()
-                 (add-hook ',hook #',function nil t))))
+                 (add-hook ',hook ,(setup-ensure-function function) nil t))))
   :documentation "Add FUNCTION to HOOK only in buffers of the current mode."
   :debug '(symbolp sexp)
   :repeatable t)
@@ -555,7 +555,7 @@ See `advice-add' for more details."
     `(unless (package-installed-p ',package)
        ,(setup-quit)))
   :documentation "If package is not installed, stop evaluating the body.
-This macro can be used as HEAD, and it will replace itself with
+This macro can be used as NAME, and it will replace itself with
 the first PACKAGE."
   :repeatable t
   :shorthand #'cadr)
@@ -565,7 +565,7 @@ the first PACKAGE."
     `(unless (featurep ',feature)
        ,(setup-quit)))
   :documentation "If FEATURE is not available, stop evaluating the body.
-This macro can be used as HEAD, and it will replace itself with
+This macro can be used as NAME, and it will replace itself with
 the first PACKAGE."
   :repeatable t
   :shorthand #'cadr)
@@ -586,11 +586,12 @@ the first PACKAGE."
 
 (setup-define :load-from
   (lambda (path)
-    `(if (file-exists-p ,path)
-         (add-to-list 'load-path (expand-file-name ,path))
-       ,(setup-quit)))
+    `(let ((path* (expand-file-name ,path)))
+       (if (file-exists-p path*)
+           (add-to-list 'load-path path*)
+         ,(setup-quit))))
   :documentation "Add PATH to load path.
-This macro can be used as HEAD, and it will replace itself with
+This macro can be used as NAME, and it will replace itself with
 the nondirectory part of PATH.
 If PATH does not exist, abort the evaluation."
   :shorthand (lambda (args) (intern
@@ -599,7 +600,7 @@ If PATH does not exist, abort the evaluation."
 
 (setup-define :file-match
   (lambda (pat)
-    `(add-to-list 'auto-mode-alist (cons ,pat ,(setup-get 'mode))))
+    `(add-to-list 'auto-mode-alist (cons ,pat ',(setup-get 'mode))))
   :documentation "Associate the current mode with files that match PAT."
   :debug '(form)
   :repeatable t)
